/*

 DC Receiver controller Timing & Interrupt routines

 OH2NLT / Juha Niinikoski 12.01.2005

 Code size optimization 18.01.2004
 New Rotary Encoder logic 20.01.2005
 Encoder debounce filter 03.01.2005

*/

// delays calculated for 20 MHz clock


static char oldpw;						// encoder old state
static char pw;							// encoder current state
static char pw_change;					// state machine variable

static unsigned char old_portb_copy;	// old encoder bits for debounce

/* Init timers & interrupt system */

void Init_timers()
	{
//	TMR1ON = 0;		// stop TMR1, TMR1 is reserved for delays and waits
//	TMR1CS = 0;		// internal clock
//	T1CKPS0 = 0;	// prescaler = 4 ==> TMR1 Clock = 1,25 MHz (800ns tick)
//	T1CKPS1 = 1;
	T1CON = 0x20;	// same as above, but fewer with instructions

	PR2 = 249;		// Generate 1 ms IRQ. 20 MHz / 4 = 5 MHz / 4 / 250 / 5 = 1 mS
	T2CON = 0x21;	// Prescaler = 4 = 1250kHz clock for TMR2, postscaler = 5
	TMR2ON = 1;		// start TIMER2
	TMR2IE = 1;		// enable TMR2 IRQ
		 
	oldpw = 3;

	PEIE = 1;		// enable peripheral IRQs
	GIE = 1;		// enable global IRQs
	}

/* IRQ handlers, TMR2 is only source */
/* TMR2 general timing IRQ */

static void interrupt tmr2_int(void)
	{
	TMR2IF = 0;						// reset IRQ flag

/* Pulse wheel logic */

	TRISB = TRISB_READ;				// change PORTB to inputs, check main loop routines !
	if(ms_timer != 0)
		ms_timer--;					// do 1ms wait timer

	asm("nop");						// few us capacitor (bus) load time
	asm("nop");
	asm("nop");						// let signal stabilize
	portb_copy = PORTB;				// read & save port bits

	if(portb_copy == old_portb_copy) // debounce filter
		pw = (portb_copy >> 6) & 3;	 // shift RB7, RB6 down & mask

	old_portb_copy = portb_copy;

	TRISB = TRISB_INIT;				// reset PORTB directions to its initial state

	if(pw != oldpw)					// first generation logic
		{
		pw_change = pw ^ oldpw;		// filter changed bit

// if you have High / Low dent encoder leave both sections active

		if(oldpw == 3)				// High at dents type encoder
			{
			if(pw_change == 1)
				{
				pwctr++;
				pwchange = 1;
				}
			if(pw_change == 2)
				{
				pwctr--;
				pwchange = 1;
				}		
			}

		if(oldpw == 0)				// Low at dents type encoder
			{
			if(pw_change == 1)
				{
				pwctr++;
				pwchange = 1;
				}
			if(pw_change == 2)
				{
				pwctr--;
				pwchange = 1;
				}		
			}

		oldpw = pw;
		}

# if 0
	switch(oldpw)				// Encoder with case, not tested
		{
		case 0:
		if(pw == 1)
			pwctr++;
		else
			pwctr--;
		break;

		case 1:
		if(pw == 3)
			pwctr++;
		else
			pwctr--;
		break;

		case 2:
		if(pw == 0)
			pwctr++;
		else
			pwctr--;
		break;

		case 3:
		if(pw == 2)
			pwctr++;
		else
			pwctr--;
		break;
		}

	oldpw = pw;					// update state
	pwchange = 1;				// set changed flag
#endif

	}



/* Timer1 is used for uS delay and wait generation */
/* Set wait. Max wait (65535) 52400 uS, TMR0IF = expiration */

void set_timeout_u( int ticks )	
	{
	union
		{
		int t1;
		unsigned char t1regs[2];
		} timex;

	timex.t1 = 0xFFFF - ticks;

	TMR1ON = 0;			// stop timer & set new value
	TMR1IF = 0;
	TMR1H = timex.t1regs[1];	// hi part must go first
	TMR1L = timex.t1regs[0];
	TMR1ON = 1;
	}


/* Microsecond delay, small delays not accurate */
/* Overdead added to requested delay about 12uS */

void delayu( unsigned unsigned int usec )
{
	set_timeout_u( usec );
	while(!TMR1IF);
}


/* Millisecond delays */

void delaym( unsigned unsigned int millisec )
{
	while(millisec != 0)
		{
		delayu(1000);
		millisec --;
		}
}	


